import java.util.*;

public class Urn {

    /**
     *  The randomizer for this Urn.
     */
    private Random randomNumberGenerator_;
    public Random randomNumberGenerator() { return randomNumberGenerator_; }
    public void randomNumberGenerator( Random aRandom ) { randomNumberGenerator_ = aRandom; }

    /**
     *  The original contents of this Urn
     */
    private Vector originalContents_;
    public Vector originalContents() { return originalContents_; }
    public void originalContents( Vector aVector ) { originalContents_ = aVector; }

    /**
     *  The current contents of this Urn .. balls are removed
     *  from here when -picked-
     */
    private Vector currentContents_;
    public Vector currentContents() { return currentContents_; }
    public void currentContents( Vector aVector ) { currentContents_ = aVector; }

    Urn() {
        this.initialize();
    }

    Urn(int ballCount, int startingNumber) {
        this();
        fillTheUrnWithUniqueBalls(startingNumber, ballCount);
    }

    Urn(int ballCount, int startingNumber, long seed) {
        this();
        this.randomNumberGenerator(new Random(seed));
        fillTheUrnWithUniqueBalls(startingNumber, ballCount);
    }
        
    // allows filling the Urn with duplicates
    Urn(int startingNumber, int endingNumber, int increment, int ballCount ) {
        this();
        fillTheUrnWithNonUniqueBalls(startingNumber, endingNumber, increment, ballCount);
    }
        
    private void initialize() {
       // this.randomNumberGenerator(new Random((long) 1234567890));
        this.randomNumberGenerator(new Random(System.currentTimeMillis()));
        this.originalContents(new Vector());
        this.currentContents(new Vector());
    }

    public void fillTheUrnWithUniqueBalls(int startingNumber, int ballCount) {
        for (int i=startingNumber; i<ballCount+startingNumber; i++) {
            this.originalContents().addElement(new Integer(i));
        }
        this.mixTheBalls();
    }
    
    public void fillTheUrnWithNonUniqueBalls(int startingNumber, int endingNumber, int increment, int ballCount) {
        
        int s = startingNumber;
        int e = endingNumber;
        
        int n = startingNumber;
        
        for (int i=0; i<ballCount; i++) {
            this.originalContents().addElement(new Integer(n));
            n += increment;
            if (n > endingNumber) n = startingNumber;
        }
        this.mixTheBalls();
    } 

    public void reload() {
        this.mixTheBalls();
    }
    
    public Integer nextNumber() {
        /*
        int theNumber = ((Integer) this.currentContents().firstElement()).intValue();
        this.currentContents().removeElementAt(0);
        return theNumber;
        */
        Integer theInt = new Integer(((Integer)this.currentContents().firstElement()).intValue());
        this.currentContents().removeElementAt(0);
        return theInt;
    }
    
    public int currentContentsCount() {
        return this.currentContents().size();
    }

    public Enumeration elements() {
        return this.currentContents().elements();
    }

    public void mixTheBalls() {
        
        Vector workVector = (Vector) this.originalContents().clone();
        this.currentContents(new Vector());
        while (workVector.size() > 0) {
            int j = workVector.size();
            int nextChoice = this.numberBetween(0, j-1);
            this.currentContents().addElement(workVector.elementAt(nextChoice));
            workVector.removeElementAt(nextChoice);
        }
    }

    public int numberBetween(int low, int high) { //inclusive

        float s = this.randomNumberGenerator().nextFloat();
        float r = s * (high - low + 1);

        if (r >= (high-low+1)) {  // i do not think this can happen
            return high;         // but just in case ..
        }

        return (((int) Math.floor (r)) + low);
    }
    
    public void displayCurrentContents() {
        boolean firstNumberAboutToPrint = true;
        Enumeration theOC = this.currentContents().elements();
        while (theOC.hasMoreElements()) {
            Integer theInt = (Integer) theOC.nextElement();
            if (firstNumberAboutToPrint) {
                System.out.print("\nUrn contents: " + theInt);
                firstNumberAboutToPrint = false;
            }
            else {
                System.out.print(", " + theInt);
            }
        }
        System.out.println();
    }
}